@@ -420,8 +420,8 @@ class BaseRequest(object):
         e = self.environ
         host = e.get('HTTP_HOST')
         if host is not None:
-            if ':' in host:
-                host, port = host.split(':', 1)
+            if ':' in host and host[-1] != ']':
+                host, port = host.rsplit(':', 1)
             else:
                 url_scheme = e['wsgi.url_scheme']
                 if url_scheme == 'https':
@@ -442,8 +442,8 @@ class BaseRequest(object):
         url = scheme + '://'
         host = e.get('HTTP_HOST')
         if host is not None:
-            if ':' in host:
-                host, port = host.split(':', 1)
+            if ':' in host and host[-1] != ']':
+                host, port = host.rsplit(':', 1)
             else:
                 port = None
         else:
@@ -674,8 +674,8 @@ class BaseRequest(object):
         .. code-block:: python
 
            domain = request.host
-           if ':' in domain:
-               domain = domain.split(':', 1)[0]
+           if ':' in domain and domain[-1] != ']': # Check for ] because of IPv6
+               domain = domain.rsplit(':', 1)[0]
 
         This will be equivalent to the domain portion of the ``HTTP_HOST``
         value in the environment if it exists, or the ``SERVER_NAME`` value in
@@ -687,8 +687,8 @@ class BaseRequest(object):
         value use :meth:`webob.request.Request.host` instead.
         """
         domain = self.host
-        if ':' in domain:
-            domain = domain.split(':', 1)[0]
+        if ':' in domain and domain[-1] != ']':
+            domain = domain.rsplit(':', 1)[0]
         return domain
 
     @property
@@ -902,14 +902,25 @@ class BaseRequest(object):
         if clen is not None and clen != 0:
             return True
         elif clen is None:
-            # rely on the special flag
-            return self.environ.get('webob.is_body_readable', False)
+            # Rely on the special flag that signifies that either Chunked
+            # Encoding is allowed (and works) or we have replaced
+            # self.body_file with something that is readable and EOF's
+            # correctly.
+            return self.environ.get(
+                'wsgi.input_terminated',
+                # For backwards compatibility, we fall back to checking if
+                # webob.is_body_readable is set in the environ
+                self.environ.get(
+                    'webob.is_body_readable',
+                    False
+                )
+            )
 
         return False
 
     @is_body_readable.setter
     def is_body_readable(self, flag):
-        self.environ['webob.is_body_readable'] = bool(flag)
+        self.environ['wsgi.input_terminated'] = bool(flag)
 
     def make_body_seekable(self):
         """
@@ -1638,8 +1649,9 @@ def _encode_multipart(vars, content_type, fout=None):
         w(b'--')
         wt(boundary)
         w(CRLF)
-        assert name is not None, 'Value associated with no name: %r' % value
-        wt('Content-Disposition: form-data; name="%s"' % name)
+        wt('Content-Disposition: form-data')
+        if name is not None:
+            wt('; name="%s"' % name)
         filename = None
         if getattr(value, 'filename', None):
             filename = value.filename
@@ -1718,12 +1730,15 @@ class Transcoder(object):
 
     def transcode_fs(self, fs, content_type):
         # transcode FieldStorage
-        if PY3:
+        if not PY3:
             def decode(b):
-                return b
+                if b is not None:
+                    return b.decode(self.charset, self.errors)
+                else:
+                    return b
         else:
             def decode(b):
-                return b.decode(self.charset, self.errors)
+                return b
 
         data = []
         for field in fs.list or ():
